###############################################################################
# KoXo MSol
# 01/07/2022
###############################################################################
function connect_msol {
    Param([string] $MyName, $MyPassword)
    $Cred = get_cred $MyName $MyPassword
    Connect-MsolService -Credential $Cred -ErrorAction SilentlyContinue
    if ([String]::IsNullorEmpty($error[0])) {
        $S = "Connection was successful to Microsoft Online Services"
        log_message "OK" $S
        Return $true
    } 
    else {
        $global:total_errors_count++
        $S = "Connection was unsuccessful to Microsoft Online Services || $($error[0])"
        log_message "ERROR" $S
        # Display error
        Write-host $S -foregroundcolor "red"
        Write-host "Press 'Enter' to continue ..."
        Read-Host   
        return $false
    }   
}
###############################################################################
# Get and display the default MSol domain
# (Get-MsolDomain |  Where {$_.IsDefault -eq $true}).Name
function get_msol_default_domain {
    $MSolDefaultDomain = ""
    $MSolDomains = Get-MsolDomain -ErrorAction SilentlyContinue
    if ($MSolDomains -ne $null) {
        Write-Host "MSol domain(s): "
        ForEach ($MSolDomain in $MSolDomains) {
            if ($MSolDomain.IsDefault -eq $true) { 
                Write-Host "$($MSolDomain.Name) [Default]" -ForegroundColor Yellow
                $MSolDefaultDomain = $MSolDomain.Name
            }
            else { Write-Host $MSolDomain.Name }
        }
    }
    return $MSolDefaultDomain
} 
###############################################################################
function get_tenant_name {
    $sku = Get-MsolAccountSku
    if ($sku.count -gt 0) {
        return $sku[0].AccountSkuId.Split(":")[0]
    }
    else {
        return ""
    }
}
###############################################################################
function msol_user_exists {
    Param([string] $Email)
    $User = Get-MSolUser -UserPrincipalName $Email -ErrorAction SilentlyContinue
    return (![string]::IsNullOrEmpty($User))
}
###############################################################################
function get_msoluser {
    $Error.Clear()
    $sku = Get-MSoluser -All -ErrorAction SilentlyContinue
    return $sku
}
###############################################################################
function msoluserlicensed
{
$Error.Clear()
$list=Get-MSoluser -ErrorAction SilentlyContinue | Sort-Object -Property DisplayName | Where {$_.Licenses.Count -gt 0} | Select DisplayName,UserPrincipalName,Licenses | Format-Table -AutoSize
return $list
}
###############################################################################
function user_has_at_least_one_license {
    Param([string] $Email)
    $User = Get-MSolUser -UserPrincipalName $Email -ErrorAction SilentlyContinue
    return (![string]::IsNullOrEmpty($User.Licenses))
}
###############################################################################
function user_has_license($Email, $LicensePack) {
    # Param([string] $email,$LicensePack)
    $User = Get-MSolUser -UserPrincipalName $Email -ErrorAction SilentlyContinue
    $Res = $false
    If ($User.Licenses.count -gt 0) {
        ForEach ($License In $User.Licenses) {
			# If ($License.AccountSku.SkuPartNumber.ToLower() -eq $LicensePack.ToLower()) {
            If ($License.AccountSkuId.ToLower() -eq $LicensePack.ToLower()) {
                $Res = $true;
            }
        }
    }
    return $Res
}
###############################################################################
function user_licenses {
    Param([string] $Email)
    $User = Get-MSolUser -UserPrincipalName $Email -ErrorAction SilentlyContinue
    $Res = ""
    If ($User.Licenses.count -gt 0) {
        ForEach ($License In $User.Licenses) {
            $Res = $Res + " " + $License.AccountSkuId
        }
    }
    return $Res.Trim()
}
###############################################################################
function get_msolaccountsku {
    $Error.Clear()
    $sku = Get-MSolAccountSku -ErrorAction SilentlyContinue
    return $sku
}
###############################################################################
function license_pack_domain {
    $sku = Get-MsolAccountSku
    if ($sku.count -gt 0) {
        return $sku[0].AccountSkuId.Split(":")[0]
    }
    else {
        return ""
    }
}
###############################################################################
function display_licenses {
    Param($UserDisplayName, $UserEmail)
    $error.Clear()
    $Licenses = (Get-MsolUser -UserPrincipalName $UserEmail -ErrorAction SilentlyContinue).Licenses
    if ($Licenses.Count -gt 0) {
        $S = "Found license packs :"
        Write-Host $S
        log_message "OK" $S
        ForEach ($License in $Licenses) {
            $S = "License pack : " + $License.AccountSkuId
            Write-Host $S
            log_message "OK" $S
        }
    }
}
###############################################################################
function remove_user_license_pack {
    Param($UserDisplayName, $UserEmail, $LicensePack)
    $error.Clear()
    Set-MSolUserLicense -UserPrincipalName $UserEmail -RemoveLicenses $LicensePack -ErrorAction SilentlyContinue
    if ([String]::IsNullorEmpty($error[0])) {
        $S = "Successfully remove license pack ""$($LicensePack)"" for ""$($UserDisplayName)"" ""($($UserEmail))"""
        write-host $S -foregroundcolor "white"
        log_message "OK" $S
    }
    else {
        $global:errors_count++
        $S = "Removing license pack ""$($LicensePack)"" for ""$($UserDisplayName)"" ""($($UserEmail))"". Error = $($error[0])"
        log_message "ERROR" $S
    }
}
###############################################################################
function set_location_user {
    Param($UserDisplayName, $UserEmail, $UsageLocation)
    $error.Clear()
    Set-MSolUser -UserPrincipalName $UserEmail -UsageLocation $UsageLocation -ErrorAction SilentlyContinue
    if ([String]::IsNullorEmpty($error[0])) {
        $S = "Successfully add location ""$($UsageLocation)"" for ""$($UserDisplayName)"" ($($UserEmail))"
        log_message "OK" $S
    }
    else {
        $global:errors_count++
        $S = "Adding location ""$($UserLocation)"" for ""$($UserDisplayName)"" ($($UserEmail)). Error = $($error[0])"
        log_message "ERROR" $S
    }
}
###############################################################################
function add_license_pack_user {
    Param($UserDisplayName, $UserEmail, $LicensePack)
    $error.Clear()
    Set-MSolUserLicense -UserPrincipalName $UserEmail -AddLicenses $LicensePack -ErrorAction SilentlyContinue
    if ([String]::IsNullorEmpty($error[0])) {
        $S = "Successfully add license pack ""$($LicensePack)"" for ""$($UserDisplayName)"" ($($UserEmail))"
        write-host $S -foregroundcolor "white"
        log_message "OK" $S
    }
    else {
        $global:errors_count++
        $S = "Adding license pack ""$($LicensePack)"" for ""$($UserDisplayName)"" ($($UserEmail)). Error = $($error[0])"
        log_message "ERROR" $S
    }
}
###############################################################################
function disable_user_plans {
    Param($UserDisplayName, $UserEmail, $LicensePack, $DisabledPlans)
    $error.Clear()
    if (![String]::IsNullOrEmpty($DisabledPlans)) {
        $ArrayDisabledPlans = $DisabledPlans.Split(",", [System.StringSplitOptions]::RemoveEmptyEntries)
        $ServicePlan = New-MsolLicenseOptions -AccountSkuId $LicensePack -DisabledPlans $ArrayDisabledPlans
        $error.Clear()
        Set-MsolUserLicense -UserPrincipalName $UserEmail -LicenseOptions $ServicePlan
        if ([String]::IsNullorEmpty($error[0])) {
            $S = "Successfully disable plans ""$($DisabledPlans)"" for ""$($UserDisplayName)"" ($($UserEmail))"
            write-host $S -foregroundcolor "white"
            log_message "OK" $S
        }
        else {
            $global:errors_count++
            $S = "Disable plans ""$($DisabledPlans)"" for ""$($UserDisplayName)"" ($($UserEmail)). Error = $($error[0])"
            log_message "ERROR" $S
        }
    }
}
###############################################################################
function check_user_licenses_and_plans {
    Param($UserDisplayName, $UserEmail)
    $Licenses = (Get-MSolUser -UserPrincipalName $UserEmail -ErrorAction SilentlyContinue).Licenses
    if ([String]::IsNullorEmpty($error[0])) {
        $S = "Details : "
        Write-Host $S
        log_message "INFO" $S
        ForEach ($License in $Licenses) {
            $S = "Pack : " + $License.AccountSkuId
            Write-Host $S -foregroundcolor cyan 
            log_message "INFO" $S
            $ServicesStatus = $License.ServiceStatus
            ForEach ($ServiceStatus In  $ServicesStatus) {
                $S = "Plan : " + $ServiceStatus.ServicePlan.ServiceName + " (" + $ServiceStatus.ServicePlan.ServiceType + ") [" + $ServiceStatus.ProvisioningStatus + "]"
                if ($ServiceStatus.ProvisioningStatus -eq "Success") {
                    Write-Host $S -foregroundcolor "green"
                }
                else {
                    if ($ServiceStatus.ProvisioningStatus -eq "Disabled") {
                        Write-Host $S -foregroundcolor "red"
                    }
                    else {
                        Write-Host $S -foregroundcolor "yellow"
                    }
                }
                log_message "INFO" $S
            }
        } 
    }
    else {
        $global:errors_count++
        $S = "Checking licenses ""$($LicensePack)"" for ""$($UserDisplayName)"" ($($UserEmail)). Error = $($error[0])"
        log_message "ERROR" $S
    }
}
###############################################################################
function set_password_options {
    Param([string] $UserDisplayName, $UserEmail, $StrongPasswordRequired, $PasswordNeverExpires)
    if ($StrongPasswordRequired -isnot [Boolean]) { $StrongPasswordRequired = $false }
    if ($PasswordNeverExpires -isnot [Boolean]) { $PasswordNeverExpires = $true }
    $error.Clear()
    Set-MsolUser -UserPrincipalName $UserEmail -StrongPasswordRequired $StrongPasswordRequired -PasswordNeverExpires $PasswordNeverExpires -ErrorAction SilentlyContinue
    if ([String]::IsNullorEmpty($error[0])) {
        $S = "Successfully configure password policy for ""$($UserDisplayName)"" ($($UserEmail))"
        log_message "OK" $S
    }
    else {
        $global:errors_count++
        $S = "Configuring password policy for ""$($UserDisplayName)"" ($($UserEmail)). Error = $($error[0])"
        log_message "ERROR" $S
    }
}
###############################################################################
function display_user_licenses {
    Param($UserDisplayName, $UserEmail)
    $error.Clear()
    $Licenses = (Get-MsolUser -UserPrincipalName $UserEmail -ErrorAction SilentlyContinue).Licenses
    if ($Licenses.Count -gt 0) {
        $S = "Found license packs :"
        Write-Host $S
        log_message "INFO" $S
        ForEach ($License in $Licenses) {
            $S = "License pack : " + $License.AccountSkuId
            Write-Host $S
            log_message "INFO" $S
        }
    }
}
###############################################################################
function get_koxo_security_group {
    Param([string] $grp)
    $error.Clear()
    $Group = Get-MsolGroup -SearchString $grp -GroupType "Security" | Where { $_.Description -cmatch "[KOXO]" }
    return $Group
}
###############################################################################
function licenses_info {
    Param([string] $usr)
    $Error.Clear()
    $crlf = "`r`n"
    if (msol_user_exists($usr)) {
        $Prefix = license_pack_domain
        $Licenses = (Get-MSolUser -UserPrincipalName $UserEmail -ErrorAction SilentlyContinue).Licenses
        if ([String]::IsNullorEmpty($error[0])) {
            ForEach ($License in $Licenses) {
                $Infos = $Infos + "Pack : " + $License.AccountSkuId + $crlf
                $ServicesStatus = $License.ServiceStatus
                ForEach ($ServiceStatus In  $ServicesStatus) {
                    $Infos = $Infos + "Plan : " + $ServiceStatus.ServicePlan.ServiceName + " (" + $ServiceStatus.ServicePlan.ServiceType + ") [" + $ServiceStatus.ProvisioningStatus + "]" + $crlf
                }
            }
        }
    } 
    else {
        $Infos = "MSol User $($usr) not found !" + $crlf + $crlf
    }
    return $Infos + $crlf
}
###############################################################################
function remove_msol_user {
    Param($UserDisplayName, $UserEmail)
    $error.Clear()
    Remove-MsolUser -UserPrincipalName $UserEmail -ErrorAction SilentlyContinue -Force
    if ([String]::IsNullorEmpty($error[0])) {
        $S = "Successfully delete mailbox for $($UserDisplayName) ($($UserEmail))"
        log_message "OK" $S
    }
    else {
        $global:errors_count++
        $S = "Deleting mailbox for $($UserDisplayName) ($($UserEmail)). Error = $($error[0])"
        log_message "ERROR" $S
    }
}
###############################################################################
function change_user_password {
    Param($UserEmail, $UserPassword, $UserDisplayName, $UserMustChangePassword, $StrongPasswordRequired, $PasswordNeverExpires)
    $error.Clear()
    if ($UserMustChangePassword -isnot [Boolean]) { $UserMustChangePassword = $false }
    if ($StrongPasswordRequired -isnot [Boolean]) { $StrongPasswordRequired = $false }
    if ($PasswordNeverExpires -isnot [Boolean]) { $PasswordNeverExpires = $true }
    $error.Clear()
    Set-MsolUser -UserPrincipalName $UserEmail -StrongPasswordRequired $StrongPasswordRequired -PasswordNeverExpires $PasswordNeverExpires -ErrorAction SilentlyContinue
    if ([String]::IsNullorEmpty($error[0])) {
        $S = "Successfully configure password policy for ""$($UserDisplayName)"" ""$($UserEmail)"""
        log_message "OK" $S
    }
    else {
        $global:errors_count++
        $S = "Configuring password policy for ""$($UserDisplayName)"" ""$($UserEmail)"". Error = $($error[0])"
        log_message "ERROR" $S
    }
    Set-MsolUserPassword -UserPrincipalName $UserEmail -NewPassword $UserPassword -ForceChangePassword $UserMustChangePassword -ErrorAction SilentlyContinue
    if ([String]::IsNullorEmpty($error[0])) {
        $S = "Successfully change password for ""$($UserDisplayName)"" ""$($UserEmail)"""
        log_message "OK" $S
    }
    else {
        $global:errors_count++
        $S = "Changing password for ""$($UserDisplayName)"" ""$($UserEmail)"". Error = $($error[0])"
        log_message "ERROR" $S
    }
}
###############################################################################
function create_security_group {
    Param($GroupName, 
        $GroupDescription,
        $GroupMembers)
    $error.Clear()
    $Group = New-MsolGroup -DisplayName $GroupName -Description ([string]::Format("[KOXO] {0}", $GroupDescription))
    if ([String]::IsNullorEmpty($error[0])) {
        $S = "Successfully create security group ""$($GroupName)"" Description=""$($GroupDescription)"" (Group Id=""$($Group.ObjectId)"")"
        log_message "OK" $S
        # Members
        $Members = $GroupMembers.Split(",", [System.StringSplitOptions]::RemoveEmptyEntries)
        ForEach ($Member in $Members) {
            if (![String]::IsNullorEmpty($Member)) {                
                $error.Clear()
                $User = Get-MsolUser -UserPrincipalName $Member
                if (![string]::IsNullOrEmpty($User)) {
                    Add-MsolGroupMember -GroupObjectId $Group.ObjectId -GroupMemberType "User" -GroupMemberObjectId $User.ObjectId   
                    if ([String]::IsNullorEmpty($error[0])) {
                        $S = "Successfully add member ""$($Member)"" for ""$($GroupName)"" (User Id=""$($User.ObjectId)"")"
                        log_message "OK" $S
                        write-host $S
                    }
                    else {
                        $global:errors_count++
                        $S = "Adding member ""$($Member)"" for ""$($GroupName)"". Error = $($error[0])"
                        log_message "ERROR" $S                  
                    }
                }   
                else {
                    $global:errors_count++
                    $S = "Adding member ""$($Member)"" for ""$($GroupName)"". Error = $($error[0])"
                    log_message "ERROR" $S                  
                }   
            }   
            else {
                $global:errors_count++
                $S = "Adding member ""$($Member)"" for ""$($GroupName)"". Error = $($error[0])"
                log_message "ERROR" $S                  
            }
        }
    }
    else {
        $global:errors_count++
        $S = "Creating Security group ""$($GroupName)"" Description=""$($GroupDescription)"". Error = $($error[0])"
        log_message "ERROR" $S
    }
}
###############################################################################
function delete_security_group {
    Param($GroupId,
        $GroupName, 
        $GroupDescription)
    $error.Clear()
    $Group = Remove-MsolGroup -ObjectId $GroupId -Force
    if ([String]::IsNullorEmpty($error[0])) {
        $S = "Successfully delete security group ""$($GroupName)"" Description=""$($GroupDescription)"", Id=""$($GroupId)"""
        log_message "OK" $S
    }
    else {
        $global:errors_count++
        $S = "Deleting distribution group ""$($GroupName)"" Description=""$($GroupDescription)"", Id=""$($GroupId)"". Error = $($error[0])"
        log_message "ERROR" $S
    }
}
###############################################################################
function modify_security_group {
    Param($GroupName, 
        $GroupDescription,
        $GroupMembers)
    $error.Clear()
    $SecurityGroup = get_koxo_security_group($GroupName)
    $NewDescription = [string]::Format("[KOXO] {0}", $GroupDescription)
    if ($SecurityGroup.Description -ne $NewDescription) {
        $NewDescription = [string]::Format("[KOXO] {0}", $GroupDescription)
        $error.Clear()
        Set-MsolGroup -ObjectId $SecurityGroup.ObjectId -Description $NewDescription
        if ([String]::IsNullorEmpty($error[0])) {
            $S = "Successfully update security group ""$($GroupName)"" Description=""$($NewDescription)"", (Group Id=""$($SecurityGroup.ObjectId)"")"
            log_message "OK" $S
        }
        else {
            $global:errors_count++
            $S = "Updating security group ""$($GroupName)"" Description=""$($GroupDescription)"". Error = $($error[0])"
            log_message "ERROR" $S
        }
    }
    $MembersSecu = Get-MsolGroupMember -GroupObjectId $SecurityGroup.ObjectId    
    $DesiredMembers = $GroupMembers.Split(",", [System.StringSplitOptions]::RemoveEmptyEntries)
    $MembersToAdd = New-Object System.Collections.ArrayList
    $MembersToRemove = New-Object System.Collections.ArrayList
    ForEach ($MemberSecu in $MembersSecu) {
        $index = $MembersToRemove.Add($MemberSecu.EmailAddress)
    }
    ForEach ($DesiredMember in $DesiredMembers) {
        $index = $MembersToAdd.Add($DesiredMember)
    }
    ForEach ($SecuMember in $MembersSecu) {
        $index = $MembersToAdd.IndexOf($SecuMember.EmailAddress)
        if ($index -gt -1) { $MembersToAdd.Remove($SecuMember.EmailAddress) }
    }
    ForEach ($MyMember in $MembersSecu) {
        $index = $DesiredMembers.IndexOf($MyMember.EmailAddress)
        if ($index -gt -1) { $MembersToRemove.Remove($MyMember.EmailAddress) }
    }
    ForEach ($Email in $MembersToRemove) {
        Write-Host "Removing member : $($Email)" -ForegroundColor Yellow
        $error.Clear()
        $Member = Get-MsolUser -UserPrincipalName $Email
        if ([String]::IsNullorEmpty($error[0])) {
            Remove-MsolGroupMember -GroupObjectId $SecurityGroup.ObjectId -GroupMemberObjectId $Member.ObjectId
            # Gestion erreurs ....
            if ([String]::IsNullorEmpty($error[0])) {
                $S = "Successfully remove member ""$($Member.DisplayName)"", Email : ""$($Member.UserPrincipalName)"",  User Id : ""$($Member.ObjectId)"" from security group ""$($SecurityGroup.DisplayName)"""
                log_message "OK" $S
            }
            else {
                $global:errors_count++
                $S = "Removing member ""$($Member.DisplayName)"", Email : ""$($Member.UserPrincipalName)"" from security group ""$($SecurityGroup.DisplayName)"". Error = $($error[0])"
                log_message "ERROR" $S
            }
        }
        else {        	
            $global:errors_count++
            $S = "Removing member ""$($Email)"" from security group ""$($SecurityGroup.DisplayName)"". Error = $($error[0])"
            log_message "ERROR" $S
        }   
    }
    ForEach ($Email in $MembersToAdd) {
        Write-Host "Adding member : $($Email)" -ForegroundColor Yellow	
        $error.Clear()
        $Member = Get-MsolUser -UserPrincipalName $Email
        if ([String]::IsNullorEmpty($error[0])) {
            Add-MsolGroupMember -GroupObjectId $SecurityGroup.ObjectId -GroupMemberObjectId $Member.ObjectId
            # Gestion erreurs ....
            if ([String]::IsNullorEmpty($error[0])) {
                $S = "Successfully add member ""$($Member.DisplayName)"", Email : ""$($Member.UserPrincipalName)"", User Id : ""$($Member.ObjectId)"" from security group ""$($SecurityGroup.DisplayName)"""
                log_message "OK" $S
            }
            else {
                $global:errors_count++
                $S = "Adding member ""$($Member.DisplayName)"", Email : ""$($Member.UserPrincipalName)"" from security group ""$($SecurityGroup.DisplayName)"". Error = $($error[0])"
                log_message "ERROR" $S
            }
        }
        else {
            $global:errors_count++
            $S = "Adding member ""$($Email)"" from security group ""$($SecurityGroup.DisplayName)"". Error = $($error[0])"
            log_message "ERROR" $S
        }
    }
}