###############################################################################
# KoXo Microsoft Teams
# 01/07/2022
###############################################################################
function connect_teams {
    Param([string] $MyName, $MyPassword)
    $Cred=get_cred $MyName $MyPassword
    Connect-MicrosoftTeams -Credential $Cred -ErrorAction SilentlyContinue
    if ([String]::IsNullorEmpty($error[0])) {
        $S = "Connection was successful to Microsoft Teams"
        log_message "OK" $S
        Return $true
    } 
    else {
        $global:total_errors_count++
        $S = "Connection was unsuccessful to Microsoft Teams || $($error[0])"
        log_message "ERROR" $S
        # Display error
        Write-host $S -foregroundcolor "red"
        Write-host "Press 'Enter' to continue ..."
        Read-Host   
        Return $false
    }   
}
function teams_module_is_present {
    if (Get-Module -ListAvailable -Name MicrosoftTeams) {
        Return $True
    }
    else {
        Return $False
    }
}
###############################################################################
function team_add_users {
    Param($Team, $Users, $Role)
    If ((![String]::IsNullorEmpty($Users)) -and ($Team -ne $null) -and (![String]::IsNullorEmpty($Role))) {
        $Options = [System.StringSplitOptions]::RemoveEmptyEntries
        $ListOfUsers = $Users.Split(",", $Options)
        If ($ListOfUsers.Count -gt 0) {
            $ListOfUsers = $ListOfUsers | Select-Object -Unique
        }
        ForEach ($User In $ListOfUsers) {
            $error.Clear()
            Add-TeamUser -GroupId $Team.GroupId -User $User -Role $Role -ErrorAction SilentlyContinue
            If ([String]::IsNullorEmpty($error[0])) {
                $S = "Successfully add user ""$($User)"" as $($Role.ToLower()) to team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
                Write-host $S -foregroundcolor "Green"
                log_message "OK" $S
            }
            else {
                $global:errors_count++
                $S = "Adding user ""$($User)"" as $($Role.ToLower()) to team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
                Write-host $S -foregroundcolor "Red"
                log_message "ERROR" $S
            }
        }
    }
}
###############################################################################
function team_remove_users {
    Param($Team, $Users)
    If ((![String]::IsNullorEmpty($Users)) -and ($Team -ne $null)) {
        $Options = [System.StringSplitOptions]::RemoveEmptyEntries
        $ListOfUsers = $Users.Split(",", $Options)
        If ($ListOfUsers.Count -gt 0) {
            $ListOfUsers = $ListOfUsers | Select-Object -Unique
        }
        ForEach ($User In $ListOfUsers) {
            $error.Clear()
            Remove-TeamUser -GroupId $Team.GroupId -User $User -ErrorAction SilentlyContinue
            If ([String]::IsNullorEmpty($error[0])) {
                $S = "Successfully remove user ""$($User)"" from team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
                Write-host $S -foregroundcolor "Green"
                log_message "OK" $S
            }
            else {
                $global:errors_count++
                $S = "Removing user ""$($User)"" from team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
                Write-host $S -foregroundcolor "Red"
                log_message "ERROR" $S
            }
        }
    }
}
###############################################################################
function team_exists {
    Param($MailNickName)
    Return ((Get-Team -MailNickName $MailNickName | Where-Object {$_.MailNickName -eq $MailNickName}) -ne $null)
}
###############################################################################
function team_remove {
    Param($MailNickName)
    $Team = Get-Team -MailNickName $MailNickName | Where-Object {$_.MailNickName -eq $MailNickName} -ErrorAction SilentlyContinue
    If ($Team -ne $null) {
        $error.Clear()
        Remove-Team -GroupId $Team.GroupId -ErrorAction SilentlyContinue
        If ([String]::IsNullorEmpty($error[0])) {
            $S = "Successfully delete team ""$($MailNickName)"""
            Write-host $S -foregroundcolor "Green"
            log_message "OK" $S
        }
        else {
            $global:errors_count++
            $S = "Creating team ""$($MailNickName)"""
            Write-host $S -foregroundcolor "Red"
            log_message "ERROR" $S
        }
    }
    else {
        $global:errors_count++
        $S = "Cannot find team ""$($MailNickName)"""
        Write-host $S -foregroundcolor "Red"
        log_message "ERROR" $S
    }
}
###############################################################################
function team_channel_exists {
    Param($GroupId, $DisplayName)
    Return ((Get-TeamChannel -GroupId $GroupId | Where DisplayName -EQ $DisplayName) -ne $null)
}
###############################################################################
# Channels : channel1,channel2,channel3 ...
# Channel : Displayname | Displayname%Description | Displayname%Description%MembershipType | Displayname%Description%MembershipType%Owner
# Warning : MembershipType and Owner are nor supported on all versions of modules
function team_add_channels {
    Param($Team, $Channels)
    If (![String]::IsNullorEmpty($Channels)) {
        $Options = [System.StringSplitOptions]::RemoveEmptyEntries
        $ListOfChannels = $Channels.Split(",", $Options)
        ForEach ($Channel In $ListOfChannels) {
            $Params = @{"GroupId" = $Team.GroupID; "ErrorAction" = "SilentlyContinue" }
            $ChannelSplit = $Channel.Split("%", $Options)
            $DisplayName = $ChannelSplit[0].Trim()
            $Description = $DisplayName
            # Description in 2nd field
            If ($ChannelSplit.Count -gt 1) {
                $Description = $ChannelSplit[1]
            }
            $Params += @{"DisplayName" = $DisplayName; "Description" = $Description }
            # "Private" without owner in fields
            If ($ChannelSplit.Count -ge 3) {
                If ($ChannelSplit[2].ToLower().Trim() -eq "private") {
                    $Params += @{"MembershipType" = "Private" }
                }
            }
            # "Private" with specified owner
            If ($ChannelSplit.Count -eq 4) {
                If ($ChannelSplit[2].ToLower().Trim() -eq "private") { 
                    $Params += @{"Owner" = $ChannelSplit[3] }
                }
            }
            $error.Clear()
            $RetChannel = New-TeamChannel @Params
            If ([String]::IsNullorEmpty($error[0])) {
                $S = "Successfully add channel ""$($DisplayName)"" (Id=$($RetChannel.Id)) to team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
                Write-host $S -foregroundcolor "Green"
                log_message "OK" $S
            }
            else {
                $global:errors_count++
                $S = "Adding channel team ""$($DisplayName)"" to team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
                Write-host $S -foregroundcolor "Red"
                log_message "ERROR" $S
            }
        }
    }
}
###############################################################################
function team_remove_channel {
    Param($Team, $DisplayName)
    If (![String]::IsNullorEmpty($DisplayName)) {
        $error.Clear()
        $Params = @{"GroupId" = $Team.GroupId; "DisplayName" = $DisplayName; "ErrorAction" = "SilentlyContinue" }
        Remove-TeamChannel @Params
        If ([String]::IsNullorEmpty($error[0])) {
            $S = "Successfully remove channel ""$($DisplayName)"" from team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
            Write-host $S -foregroundcolor "Green"
            log_message "OK" $S
        }
        else {
            $global:errors_count++
            $S = "Removing channel team ""$($DisplayName)"" from team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
            Write-host $S -foregroundcolor "Red"
            log_message "ERROR" $S
        }
    }
}
###############################################################################
# Warning : Template is not supported on all versions of modules
# Warning : Template : must be "EDU_Class" or "PLC_Class"
function team_create {
    Param($DisplayName, $Description, $MailNickName, $Visibility, $Owners, $DefaultOwner, $Template, $Channels, $MSolDefaultDomain, $Office365DNSDomain)
    If (team_exists($MailNickName)) {
        $S = "Team ""$($MailNickName)"" already exists !"
        Write-Host $S -Foregroundcolor "Red"
        Log_Message "ERROR" $S
        $global:total_errors_count++
        Return $null
        Break  
    }
    If (![String]::IsNullorEmpty($Owners)) {
        $Options = [System.StringSplitOptions]::RemoveEmptyEntries
        $ListOfOwners = $Owners.Split(",", $Options)
        If ($ListOfOwners.Count -gt 0) {
            $Owner = $ListOfOwners[0]
        }
    }
    else {
        $Owner = $DefaultOwner
    }
    If ([String]::IsNullorEmpty($Description)) {
        $Description = $Name
    }
    $Params = @{"DisplayName" = $DisplayName; "MailNickName" = $MailNickName; "Owner" = $Owner; "ErrorAction" = "SilentlyContinue" }
    If (![String]::IsNullorEmpty($Description)) {
        $Params += @{"Description" = $Description }
    }
    If (![String]::IsNullorEmpty($Template)) {
        $Params += @{"Template" = $Template }
    }
    else {
        $Params += @{"Visibility" = $Visibility }
    }
    $error.Clear()
    $Team = New-Team @Params
    If ([String]::IsNullorEmpty($error[0])) {
        $S = "Successfully create team ""$($DisplayName)"" (MailNickName=$($MailNickName), Id=$($Team.GroupId))"
        Write-host $S -foregroundcolor "Green"
        log_message "OK" $S
        team_add_channels $Team $Channels
    }
    else {
        $global:errors_count++
        $S = "Creating team ""$($DisplayName)"" (MailNickName=$($MailNickName))"
        Write-host $S -foregroundcolor "Red"
        log_message "ERROR" $S
    }
    team_add_users $Team $Members "Member"
    team_add_users $Team $Owners "Owner"
    # If the domain is not the default domain
    If ($MSolDefaultDomain -ne $Office365DNSDomain) {
       # Primary SMTP Address
       $PrimarySMTPAddress = "$($MailNickName)@$($Office365DNSDomain)"
       Set-UnifiedGroup -Identity $MailNickName -PrimarySmtpAddress $PrimarySMTPAddress -ErrorAction SilentlyContinue
       Write-Host "Team Primary Smtp Address modified to : $($PrimarySMTPAddress)" -ForegroundColor Cyan
       $OldPrimarySMTPAddress = "smtp:$($MailNickName)@$($MSolDefaultDomain)"
       Set-UnifiedGroup -Identity $MailNickName -EmailAddresses @{remove=$OldPrimarySMTPAddress} -ErrorAction SilentlyContinue
       $EmailAddresses = (Get-UnifiedGroup -Identity $MailNickName).EmailAddresses
       Write-Host "Email addresses: " -ForegroundColor Cyan
       ForEach ($EmailAddress in $EmailAddresses) {
         Write-Host $EmailAddress -ForegroundColor Cyan
       }
    }
    Return $Team
}
###############################################################################
# Modify : DisplayName, Description, Visibility, Owners, Members, Channels
function team_modify {
    Param($DisplayName, $Description, $MailNickName, $Visibility, $Owners, $DefaultOwner, $Members, $Channels, $ResetTeam, $MSolDefaultDomain, $Office365DNSDomain)
    If (![String]::IsNullorEmpty($Description)) {
        $Description = $DisplayName;
    }
    $Team = Get-Team -MailNickName $MailNickName | Where-Object {$_.MailNickName -eq $MailNickName} -ErrorAction SilentlyContinue
    If ($Team -ne $null) {
        If ($ResetTeam) {
            $S = "Reset mode enabled"
            Write-host $S -foregroundcolor "Yellow"
            log_message "INFO" $S
        }
        # Modify team
        $Params = @{"GroupId" = $Team.GroupId; "DisplayName" = $DisplayName; "Description" = $Description; "ErrorAction" = "SilentlyContinue" }
        If ([String]::IsNullorEmpty($Template)) { 
            If ($Team.Visibility -ne "HiddenMembership") {
                $Params += @{"Visibility" = $Visibility }
            }
        }
        $error.Clear()
        $Ret = Set-Team @Params
        If ([String]::IsNullorEmpty($error[0])) {
            $S = "Modifying team ""$($DisplayName)"" (GroupID=$($Team.GroupId), MailNickName=$($MailNickName))"
            Write-host $S -foregroundcolor "Green"
            log_message "OK" $S
        }
        else {
            $global:errors_count++
            $S = "Modifying team ""$($DisplayName)"" (MailNickName=$($MailNickName))"
            Write-host $S -foregroundcolor "Red"
            log_message "ERROR" $S
        }
        # Synchronize Channels
        $ListOfChannels = $Channels.Split(",", [System.StringSplitOptions]::RemoveEmptyEntries)
        $Params = @{"GroupId" = $Team.GroupId; "ErrorAction" = "SilentlyContinue" }
        $error.Clear()
        $TeamChannels = Get-TeamChannel @Params
        ForEach ($TempChannel in $ListOfChannels) {
            # Add channel if necessary
            If ($TeamChannels.DisplayName.IndexOf($TempChannel.Split('%')[0]) -eq -1) {
                team_add_channels $Team $TempChannel 
            }
        }
        If ($ResetTeam) {
            $ListOfChannels2 = New-Object System.Collections.Generic.List[string]
            ForEach ($TempChannel in $ListOfChannels) {
                $ListOfChannels2.Add($TempChannel.Split('%')[0])
            }
            ForEach ($TeamChannel in $TeamChannels) {
                # Remove a channel if necessary, and not "General"
                $ChannelDisplayName = $TeamChannel.DisplayName
                If (($ListOfChannels2.IndexOf($ChannelDisplayName) -eq -1) -and ($ChannelDisplayName.ToLower() -ne "general")) {
                    team_remove_channel $Team $ChannelDisplayName
                }
            }
        }
        # Synchronize members & owners
        $TeamsUsers = Get-TeamUser -GroupId $Team.GroupId -ErrorAction SilentlyContinue
        If ($TeamsUsers -ne $null) {
            # Existing members and owners
            $ExistingOwners = New-Object System.Collections.Generic.List[string]
            $ExistingMembers = New-Object System.Collections.Generic.List[string]
            ForEach ($TeamUser in $TeamsUsers) {
                If ($TeamUser.Role -eq "Member") {
                    $ExistingMembers.Add($TeamUser.User.ToLower())
                }
                else {
                    $ExistingOwners.Add($TeamUser.User.ToLower())
                }
            }
            $TotalOwners = $ExistingOwners.Count
            # Members and owners
            $Owners = $Owners.ToLower()
            $ListOfOwners = $Owners.Split(",", [System.StringSplitOptions]::RemoveEmptyEntries)
            If ($ListOfOwners.Count -gt 0) {
                $ListOfOwners = $ListOfOwners | Select-Object -Unique
            }
            $Members = $Members.ToLower()
            $ListOfMembers = $Members.Split(",", [System.StringSplitOptions]::RemoveEmptyEntries)
            If ($ListOfMembers.Count -gt 0) {
                $ListOfMembers = $ListOfMembers | Select-Object -Unique
            }
            # Add members and owners
            ForEach ($TempUser In $ListOfMembers) {
                If ($ExistingMembers.IndexOf($TempUser) -eq -1) {
                    team_add_users $Team $TempUser "Member"
                }
            }
            ForEach ($TempUser In $ListOfOwners) {
                If ($ExistingOwners.IndexOf($TempUser) -eq -1) {
                    team_add_users $Team $TempUser "Owner"
                    $TotalOwners++
                }
            }
            If ($ResetTeam) {
                # Remove members and owners
                ForEach ($Tempuser In $ExistingMembers) {
                    If ($ListOfMembers.IndexOf($TempUser) -eq -1) {
                        team_remove_users $Team $TempUser
                    }
                }
                ForEach ($Tempuser In $ExistingOwners) {
                    If ($ListOfOwners.IndexOf($TempUser) -eq -1) {
                        If ($TotalOwners -eq 1) {
                            if ($DefaultOwner -ne $TempUser) {
                                team_add_users $Team $DefaultOwner "Owner"
                                team_remove_users $Team $TempUser
                                $TotalOwners--
                            }
                        }
                        else {
                            If ($TotalOwners -gt 1) {
                                team_remove_users $Team $TempUser
                                $TotalOwners--
                            }
                        }
                    }
                }
            }
        }
        else {
            $global:errors_count++
            $S = "Cannot read users from team ""$($MailNickName)"""
            Write-host $S -foregroundcolor "Red"
            log_message "ERROR" $S
        }
        If ($MSolDefaultDomain -ne $Office365DNSDomain) {
           # Primary SMTP Address
           $PrimarySMTPAddress = "$($MailNickName)@$($Office365DNSDomain)"
           Set-UnifiedGroup -Identity $MailNickName -PrimarySmtpAddress $PrimarySMTPAddress -ErrorAction SilentlyContinue
           Write-Host "Team Primary Smtp Address modified to : $($PrimarySMTPAddress)" -ForegroundColor Cyan
           $OldPrimarySMTPAddress = "smtp:$($MailNickName)@$($MSolDefaultDomain)"
           Set-UnifiedGroup -Identity $MailNickName -EmailAddresses @{remove=$OldPrimarySMTPAddress} -ErrorAction SilentlyContinue
           $EmailAddresses = (Get-UnifiedGroup -Identity $MailNickName).EmailAddresses
           Write-Host "Email addresses: " -ForegroundColor Cyan
           ForEach ($EmailAddress in $EmailAddresses) {
             Write-Host $EmailAddress -ForegroundColor Cyan
           }
        }
    }
    else {
        $global:errors_count++
        $S = "Cannot find team : MailNickName=""$($MailNickName)"""
        Write-host $S -foregroundcolor "Red"
        log_message "ERROR" $S
    }
	Return $Team
}
